import 'dart:typed_data';
import '../marker_handler/marker_data.dart';
import 'package:flutter/material.dart';
import 'package:syncfusion_flutter_maps/maps.dart';
import 'dart:math';

/// A class containing thickness for each subdivision of the map.
class IceThicknessModel {
  IceThicknessModel(this.subDivID, this.thickness);

  final String subDivID;
  final int thickness;
}

/// A stateful widget which contains a choropleth map.
/// The map data is fetched from the server, and the map is rendered
/// using the Syncfusion Flutter Maps library.
class ChoroplethMap extends StatefulWidget {
  const ChoroplethMap({Key? key,
    required this.relation,
    required this.measurements
  }) : super(key: key);

  final Uint8List relation;
  final List<Measurement> measurements;

  @override
  _ChoroplethMapState createState() => _ChoroplethMapState();
}

class _ChoroplethMapState extends State<ChoroplethMap> {
  late MapShapeSource mapShapeSource;
  late final MapZoomPanBehavior _zoomPanBehavior = MapZoomPanBehavior();
  List<IceThicknessModel> iceThicknessList = <IceThicknessModel>[];

  @override
  void initState() {
    super.initState();

    final Random random = Random();
    for (int i = 0; i <= 60; i++) {
      int randomNumber = random.nextInt(21); // 0 -> 20
      iceThicknessList.add(IceThicknessModel(i.toString(), randomNumber));
    }
  }

  @override
  Widget build(BuildContext context) {
    return SfMaps(
        layers: [
          MapShapeLayer(
            source: MapShapeSource.memory(
              widget.relation,
              shapeDataField: 'properties.SubDivID',
              dataCount: iceThicknessList.length,
              primaryValueMapper: (int index) => iceThicknessList[index].subDivID,
              shapeColorValueMapper: (int index) => iceThicknessList[index].thickness,
              shapeColorMappers: const [
                MapColorMapper(
                  from: null,  // Default color
                  color: Colors.grey,
                  text: 'No value',
                ),
                MapColorMapper(
                    from: 0,
                    to: 3,
                    color: Color.fromRGBO(223,169,254, 1),
                    text: '0-3'),
                MapColorMapper(
                    from: 4,
                    to: 8,
                    color: Color.fromRGBO(190,78,253, 1),
                    text: '4-8'),
                MapColorMapper(
                    from: 9,
                    to: 15,
                    color: Color.fromRGBO(167,17,252, 1),
                    text: '9-15'),
                MapColorMapper(
                    from: 16,
                    to: 20,
                    color: Color.fromRGBO(170,20,250, 1),
                    text: '16-20'),
              ],
            ),
            //color: Colors.lightBlueAccent,
            zoomPanBehavior: _zoomPanBehavior,
            strokeColor: Colors.orange,
          ),
        ],
    );
  }
}
