import geopandas as gpd
from shapely.geometry import Polygon

def get_relation(self, body_of_water: str):
    # Load GeoJSON data using geopandas
    geo_data = gpd.read_file("server/map/mjosa.geojson")

    # Filter only polygons, exclude points and other feature types to reduce response size
    polygon_data = geo_data[geo_data['geometry'].geom_type == 'Polygon']

    # Extract coordinates from polygons and create polygon objects
    polygons = [Polygon(polygon.exterior) for polygon in polygon_data['geometry']]

    if len(polygons) <= 1:  # Return if conversion to polygon fails
        print("Failed to convert to polygons")
        return

    # Divide relation into tiles
    tiles = divide_relation(polygons)

    # Convert GeoDataFrame to GeoJSON
    tiles_json = tiles.to_json()

    # Set headers
    self.send_response(200)
    self.send_header("Content-type", "application/json")
    self.end_headers()

    # Write coordinates to response object
    self.wfile.write(tiles_json.encode('utf-8'))


def divide_relation(polygons):
    # Define tile size
    tile_size = 0.1
    subdiv_id = 0
    tiles = []

    for polygon in polygons:
        x_min, y_min, x_max, y_max = polygon.bounds
        rows = int((y_max - y_min) / tile_size)
        cols = int((x_max - x_min) / tile_size)

        if rows == 0 or cols == 0:  # Skip polygons that are smaller than the division size
            continue

        for row in range(rows):
            for col in range(cols):
                tile_bbox = Polygon([  # Calculate coordinate for current place in column and row
                    (x_min + col * tile_size, y_min + row * tile_size),
                    (x_min + (col + 1) * tile_size, y_min + row * tile_size),
                    (x_min + (col + 1) * tile_size, y_min + (row + 1) * tile_size),
                    (x_min + col * tile_size, y_min + (row + 1) * tile_size)
                ])
                tiles.append({"SubDivID": subdiv_id, "geometry": tile_bbox})
                subdiv_id += 1

    if len(tiles) <= 1:  # Return empty object if division fails
        print("Failed to divide polygons into tiles")
        return []

    # Convert tiles list to a GeoDataFrame
    tiles_df = gpd.GeoDataFrame(tiles, geometry='geometry')

    return tiles_df
